import matplotlib.pyplot as plt
import numpy as np

# Data for D1, D2, D3 and IDAstar
d1 = [1.0, 1.0, 1.0, 1.0, 1.1, 1.0, 1.0, 1.0, 1.2]
d2 = [1.3, 1.4, 1.3, 1.5, 1.5, 1.9, 1.9, 1.8, 1.6]
d3 = [1.3, 1.4, 1.4, 1.5, 1.5, 1.4, 1.2, 1.2, 1.2]
idastar = [24.258, 24.254, 24.338, 24.304, 25.018, 26.376, 26.605, 26.707, 26.730]

# Convert data to numpy arrays for better handling
d1 = np.array(d1)
d2 = np.array(d2)
d3 = np.array(d3)
idastar = np.array(idastar)

# Find the index of the lowest IDAstar
min_index = np.argmin(idastar)

# Create bar width and positions for bars
bar_width = 0.25
index = np.arange(len(d1))

# Create a new plot with the same D1, D2, D3 bars but with only one IDAstar line and combo2 highlighted
fig, ax = plt.subplots(figsize=(12, 6))  # Increased width to ensure all bars fit

# Plot bars for D1, D2, D3 with faded colors
bar1 = ax.bar(index, d1, bar_width, color='lightblue', label='D1')
bar2 = ax.bar(index + bar_width, d2, bar_width, color='lightgreen', label='D2')
bar3 = ax.bar(index + 2 * bar_width, d3, bar_width, color='lightcoral', label='D3')

# Highlight combo2 with solid colors
highlight_index = 1  # Combo 2
bar1[highlight_index].set_color('blue')
bar2[highlight_index].set_color('green')
bar3[highlight_index].set_color('red')

# Adding the IDAstar as a single line plot (with no special highlighting for any combo)
ax2 = ax.twinx()  # Create a second y-axis for the IDAstar values
ax2.plot(index + bar_width, idastar, color='black', marker='o', linestyle='-', label='IDAstar (s)', linewidth=2)

# Set the y-axis range from 0 to 2.25
ax.set_ylim(0, 2.25)

# Annotate D1, D2, D3 values on top of the bars
for i in range(len(d1)):
    color = 'blue' if i == highlight_index else 'lightblue'
    ax.text(i, d1[i] + 0.05, f'{d1[i]:.1f}', ha='center', va='bottom', color=color)

    color = 'green' if i == highlight_index else 'lightgreen'
    ax.text(i + bar_width, d2[i] + 0.05, f'{d2[i]:.1f}', ha='center', va='bottom', color=color)

    color = 'red' if i == highlight_index else 'lightcoral'
    ax.text(i + 2 * bar_width, d3[i] + 0.05, f'{d3[i]:.1f}', ha='center', va='bottom', color=color)

# Labels and titles
ax.set_xlabel('Combinations', fontsize=12)
ax.set_ylabel('Values (D1, D2, D3)', fontsize=12)
ax2.set_ylabel('IDAstar (s)', fontsize=12)

ax.set_title('Comparison of D1, D2, D3 Values and IDAstar Execution Time', fontsize=14)
ax.set_xticks(index + bar_width)
ax.set_xticklabels([f'Comb {i + 1}' for i in range(len(d1))])

# Add legends for both graphs
ax.legend(loc='upper left', fontsize=12)
ax2.legend(loc='upper right', fontsize=12)

# Adjust layout to ensure nothing is cut off
plt.tight_layout()
plt.show()
