// Create a new scene
var scene = PADrend.createNewSceneRoot("new MinSG.ListNode");

// activate the scene
PADrend.selectScene(scene);

// Create the ground -----------------------------------------------------------
var vd = new Rendering.VertexDescription;
vd.appendPosition3D(); // 3xfloat (x,y,z)
vd.appendNormalByte(); // 4xbyte (x,y,z,0)
vd.appendColorRGBAByte(); // 4xbyte (r,g,b,a)

var grid = Rendering.createHexGrid(vd, 20, 20, 32, 32);

var scale = 10;
var transform = (new Geometry.Matrix4x4).scale(0.01, 0.01, 0.01); // scale matrix
Rendering.calculateNormals(grid);

var gridNode = new MinSG.GeometryNode(grid);

var stateGrid = new MinSG.MaterialState;
stateGrid.setAmbient(new Util.Color4f(0.16, 0.47, 0.19, 1));
stateGrid.setDiffuse(new Util.Color4f(0.16, 0.47, 0.19, 1));

gridNode += stateGrid;
scene += gridNode;


// Create obstacles ------------------------------------------------------------
var stateCone = new MinSG.MaterialState;
stateCone.setAmbient(new Util.Color4f(0.96, 0.47, 0.19, 1));
stateCone.setDiffuse(new Util.Color4f(0.96, 0.47, 0.19, 1));

var coneMesh = Rendering.createCone(vd, 0.1, 0.4, 4);
var geoNode = new MinSG.GeometryNode();
geoNode.moveLocal(10, 0, 20);
geoNode.scale(25);
var rot = new Geometry.Vec3();
rot.setX(50);
rot.setY(50);
rot.setZ(0.5);
var up = new Geometry.Vec3();
up.setX(0);
up.setY(1);
up.setZ(0);
geoNode.setRelRotation(rot, up);
geoNode.setMesh(coneMesh);
geoNode += stateCone;
scene += geoNode;


// Call the algorihtm ----------------------------------------------------------
//Pathfinder.disableOutput();
//Pathfinder.disableOBJ();
//Pathfinder.disableLogging();
var start = new Geometry.Vec3(15, 0, 10);
//var goal = new Geometry.Vec3(10, 2.5, 20);
var goal = new Geometry.Vec3(10.1, 0, 10.0005);
var path = Pathfinder.findPath(scene, start, goal, 1);
Pathfinder.enableOutput();

if(path.count() == 0) {
	path = [start, goal];
}


// Draw the found path ---------------------------------------------------------
var stateStart = new MinSG.MaterialState;
stateStart.setAmbient(new Util.Color4f(0, 1, 0, 1));
stateStart.setDiffuse(new Util.Color4f(0, 1, 0, 1));

var startMesh = Rendering.createCube();
var startGeoNode = new MinSG.GeometryNode();
startGeoNode.moveLocal(path[0]);
startGeoNode.scale(1);
startGeoNode.setMesh(startMesh);
startGeoNode += stateStart;
scene += startGeoNode;

var stateNode = new MinSG.MaterialState;
stateNode.setAmbient(new Util.Color4f(1, 0, 0, 1));
stateNode.setDiffuse(new Util.Color4f(1, 0, 0, 1));

for(var i = 1; i < path.count() - 1; ++i) {
	var nodeMesh = Rendering.createCube();
	var nodeGeoNode = new MinSG.GeometryNode();
	nodeGeoNode.moveLocal(path[i]);
	nodeGeoNode.scale(0.25);
	nodeGeoNode.setMesh(nodeMesh);
	nodeGeoNode += stateNode;
	scene += nodeGeoNode;

	var lineMesh = Rendering.createLine(vd, path[i-1], path[i]);
	var lineGeoNode = new MinSG.GeometryNode();
	lineGeoNode.setMesh(lineMesh);
	lineGeoNode += stateNode;
	scene += lineGeoNode;
}

var nodeMesh = Rendering.createCube();
var nodeGeoNode = new MinSG.GeometryNode();
nodeGeoNode.moveLocal(path[path.count()-1]);
nodeGeoNode.scale(1);
nodeGeoNode.setMesh(nodeMesh);
nodeGeoNode += stateNode;
scene += nodeGeoNode;

var lineMesh = Rendering.createLine(vd, path[path.count()-2], path[path.count()-1]);
var lineGeoNode = new MinSG.GeometryNode();
lineGeoNode.setMesh(lineMesh);
lineGeoNode += stateNode;
scene += lineGeoNode;


// Camera ----------------------------------------------------------------------
var camera = PADrend.getDolly();
camera.setWorldOrigin(new Geometry.Vec3(5,7,8));
camera.rotateToWorldDir(new Geometry.Vec3(-220,150,-190));
