// Create a new scene
var scene = PADrend.createNewSceneRoot("new MinSG.ListNode");

// activate the scene
PADrend.selectScene(scene);

// Create the ground -----------------------------------------------------------
var vd = new Rendering.VertexDescription;
vd.appendPosition3D(); // 3xfloat (x,y,z)
vd.appendNormalByte(); // 4xbyte (x,y,z,0)
vd.appendColorRGBAByte(); // 4xbyte (r,g,b,a)

var grid = Rendering.createHexGrid(vd, 30, 30, 32, 32);

var scale = 10;
var transform = (new Geometry.Matrix4x4).scale(0.01, 0.01, 0.01); // scale matrix
 Rendering.calculateNormals(grid);

var gridNode = new MinSG.GeometryNode(grid);

var stateGrid = new MinSG.MaterialState;
stateGrid.setAmbient(new Util.Color4f(0.16, 0.47, 0.19, 1));
stateGrid.setDiffuse(new Util.Color4f(0.16, 0.47, 0.19, 1));

gridNode += stateGrid;
scene += gridNode;


// Create obstacles ------------------------------------------------------------
var stateCube = new MinSG.MaterialState;
stateCube.setAmbient(new Util.Color4f(0.96, 0.47, 0.19, 1));
stateCube.setDiffuse(new Util.Color4f(0.96, 0.47, 0.19, 1));

var cubeMesh = Rendering.createCube();
var geoNode = new MinSG.GeometryNode();
geoNode.moveLocal(3.45, 0, 10);
geoNode.scale(6);
geoNode.setMesh(cubeMesh);
geoNode += stateCube;
scene += geoNode;

var cubeMesh2 = Rendering.createCube();
var geoNode2 = new MinSG.GeometryNode();
geoNode2.moveLocal(9, 0, 12);
geoNode2.scale(6);
geoNode2.setMesh(cubeMesh2);
geoNode2 += stateCube;
scene += geoNode2;

var cubeMesh3 = Rendering.createCube();
var geoNode3 = new MinSG.GeometryNode();
geoNode3.moveLocal(15, 0, 14);
geoNode3.scale(6);
geoNode3.setMesh(cubeMesh3);
geoNode3 += stateCube;
scene += geoNode3;

var cubeMesh4 = Rendering.createCube();
var geoNode4 = new MinSG.GeometryNode();
geoNode4.moveLocal(21, 0, 16);
geoNode4.scale(6);
geoNode4.setMesh(cubeMesh4);
geoNode4 += stateCube;
scene += geoNode4;

// Call the algorithm ----------------------------------------------------------
//Pathfinder.disableOutput();
//Pathfinder.disableOBJ();
//Pathfinder.disableLogging();
var start = new Geometry.Vec3(15, 0, 5);
var goal = new Geometry.Vec3(10, 0, 25);
var path = Pathfinder.findPath(scene, start, goal, 1);
Pathfinder.enableOutput();

if(path.count() == 0) {
	path = [start, goal];
}


// Draw the found path ---------------------------------------------------------
var stateStart = new MinSG.MaterialState;
stateStart.setAmbient(new Util.Color4f(0, 1, 0, 1));
stateStart.setDiffuse(new Util.Color4f(0, 1, 0, 1));

var startMesh = Rendering.createCube();
var startGeoNode = new MinSG.GeometryNode();
startGeoNode.moveLocal(path[0]);
startGeoNode.scale(1);
startGeoNode.setMesh(startMesh);
startGeoNode += stateStart;
scene += startGeoNode;

var stateNode = new MinSG.MaterialState;
stateNode.setAmbient(new Util.Color4f(1, 0, 0, 1));
stateNode.setDiffuse(new Util.Color4f(1, 0, 0, 1));

for(var i = 1; i < path.count() - 1; ++i) {
	var nodeMesh = Rendering.createCube();
	var nodeGeoNode = new MinSG.GeometryNode();
	nodeGeoNode.moveLocal(path[i]);
	nodeGeoNode.scale(0.25);
	nodeGeoNode.setMesh(nodeMesh);
	nodeGeoNode += stateNode;
	scene += nodeGeoNode;

	var lineMesh = Rendering.createLine(vd, path[i-1], path[i]);
	var lineGeoNode = new MinSG.GeometryNode();
	lineGeoNode.setMesh(lineMesh);
	lineGeoNode += stateNode;
	scene += lineGeoNode;
}

var nodeMesh = Rendering.createCube();
var nodeGeoNode = new MinSG.GeometryNode();
nodeGeoNode.moveLocal(path[path.count()-1]);
nodeGeoNode.scale(1);
nodeGeoNode.setMesh(nodeMesh);
nodeGeoNode += stateNode;
scene += nodeGeoNode;

var lineMesh = Rendering.createLine(vd, path[path.count()-2], path[path.count()-1]);
var lineGeoNode = new MinSG.GeometryNode();
lineGeoNode.setMesh(lineMesh);
lineGeoNode += stateNode;
scene += lineGeoNode;


// Camera ----------------------------------------------------------------------
var camera = PADrend.getDolly();
camera.setWorldOrigin(new Geometry.Vec3(30,15,15));
camera.rotateToWorldDir(new Geometry.Vec3(90,90,0));
