// Create a new scene
var scene = PADrend.createNewSceneRoot("new MinSG.ListNode");

// activate the scene
PADrend.selectScene(scene);

// Create the ground -----------------------------------------------------------
var vd = new Rendering.VertexDescription;
vd.appendPosition3D(); // 3xfloat (x,y,z)
vd.appendNormalByte(); // 4xbyte (x,y,z,0)
vd.appendColorRGBAByte(); // 4xbyte (r,g,b,a)

var grid = Rendering.createHexGrid(vd, 100, 100, 32, 32);

var scale = 10;
var transform = (new Geometry.Matrix4x4).scale(0.01, 0.01, 0.01); // scale matrix
 Rendering.calculateNormals(grid);

outln(" creating ground");
var gridNode = new MinSG.GeometryNode(grid);

var stateGrid = new MinSG.MaterialState;
stateGrid.setAmbient(new Util.Color4f(0.16, 0.47, 0.19, 1));
stateGrid.setDiffuse(new Util.Color4f(0.16, 0.47, 0.19, 1));

gridNode += stateGrid;
scene += gridNode;


// Create obstacles ------------------------------------------------------------
 // Set up material for obstacles (black color as in the image)
 var stateObstacle = new MinSG.MaterialState;
 stateObstacle.setAmbient(new Util.Color4f(0, 0, 0, 1));  // Black color for obstacles
 stateObstacle.setDiffuse(new Util.Color4f(0, 0, 0, 1));

// Large rectangular and cylindrical obstacles
var obstacle1 = Rendering.createRectangle(vd, new Geometry.Rect(40, 80));  // Approximate large vertical obstacle near top-left
var geoNode1 = new MinSG.GeometryNode();
geoNode1.moveLocal(200, 0, 100);  // Position it as needed
geoNode1.setMesh(obstacle1);
geoNode1 += stateObstacle;
scene += geoNode1;

var obstacle2 = Rendering.createRectangle(vd, new Geometry.Rect(160, 60));  // Approximate large horizontal obstacle near center-right
var geoNode2 = new MinSG.GeometryNode();
geoNode2.moveLocal(500, 0, 400);  // Position it in the middle-right
geoNode2.setMesh(obstacle2);
geoNode2 += stateObstacle;
scene += geoNode2;

var obstacle3 = Rendering.createRectangle(vd, new Geometry.Rect(100, 50));  // Approximate large horizontal obstacle at the bottom-left
var geoNode3 = new MinSG.GeometryNode();
geoNode3.moveLocal(300, 0, 600);  // Position it at the bottom-left
geoNode3.setMesh(obstacle3);
geoNode3 += stateObstacle;
scene += geoNode3;

// Small obstacles
var obstacle4 = Rendering.createRectangle(vd, new Geometry.Rect(30, 60));  // Small vertical obstacle near top-left
var geoNode4 = new MinSG.GeometryNode();
geoNode4.moveLocal(100, 0, 150);  // Position it
geoNode4.setMesh(obstacle4);
geoNode4 += stateObstacle;
scene += geoNode4;

var obstacle5 = Rendering.createRectangle(vd, new Geometry.Rect(40, 20));  // Small horizontal obstacle near middle-left
var geoNode5 = new MinSG.GeometryNode();
geoNode5.moveLocal(50, 0, 400);  // Position it
geoNode5.setMesh(obstacle5);
geoNode5 += stateObstacle;
scene += geoNode5;

var obstacle6 = Rendering.createRectangle(vd, new Geometry.Rect(80, 20));  // Horizontal rectangular obstacle near bottom-right
var geoNode6 = new MinSG.GeometryNode();
geoNode6.moveLocal(700, 0, 500);  // Position near bottom-right
geoNode6.setMesh(obstacle6);
geoNode6 += stateObstacle;
scene += geoNode6;

// Approximate medium elliptical obstacle using a cube
var obstacle7 = Rendering.createCube();  // Use a cube to approximate medium-sized obstacle near center
var geoNode7 = new MinSG.GeometryNode();
geoNode7.moveLocal(400, 0, 300);  // Position it at the center
geoNode7.scale(30, 10, 30);  // Scale to make it look closer to an ellipse
geoNode7.setMesh(obstacle7);
geoNode7 += stateObstacle;
scene += geoNode7;

// Additional rectangular obstacles
var obstacle8 = Rendering.createRectangle(vd, new Geometry.Rect(50, 20));  // Long horizontal rectangle near the bottom-right
var geoNode8 = new MinSG.GeometryNode();
geoNode8.moveLocal(550, 0, 600);  // Position near bottom-right
geoNode8.setMesh(obstacle8);
geoNode8 += stateObstacle;
scene += geoNode8;

var obstacle9 = Rendering.createRectangle(vd, new Geometry.Rect(30, 10));  // Small rectangle near the bottom-left
var geoNode9 = new MinSG.GeometryNode();
geoNode9.moveLocal(300, 0, 700);  // Position near the bottom-left
geoNode9.setMesh(obstacle9);
geoNode9 += stateObstacle;
scene += geoNode9;

// Call the algorithm ----------------------------------------------------------
//Pathfinder.disableOutput();
//Pathfinder.disableOBJ();
//Pathfinder.disableLogging();
var start = new Geometry.Vec3(15, 0, 5);
var goal = new Geometry.Vec3(10, 0, 25);
var path = Pathfinder.findPath(scene, start, goal, 1);
Pathfinder.enableOutput();

if(path.count() == 0) {
	path = [start, goal];
}

outln(" Algo finished");
// Draw the found path ---------------------------------------------------------
var stateStart = new MinSG.MaterialState;
stateStart.setAmbient(new Util.Color4f(0, 1, 0, 1));
stateStart.setDiffuse(new Util.Color4f(0, 1, 0, 1));

var startMesh = Rendering.createCube();
var startGeoNode = new MinSG.GeometryNode();
startGeoNode.moveLocal(path[0]);
startGeoNode.scale(1);
startGeoNode.setMesh(startMesh);
startGeoNode += stateStart;
scene += startGeoNode;

var stateNode = new MinSG.MaterialState;
stateNode.setAmbient(new Util.Color4f(1, 0, 0, 1));
stateNode.setDiffuse(new Util.Color4f(1, 0, 0, 1));

for(var i = 1; i < path.count() - 1; ++i) {
	var nodeMesh = Rendering.createCube();
	var nodeGeoNode = new MinSG.GeometryNode();
	nodeGeoNode.moveLocal(path[i]);
	nodeGeoNode.scale(0.25);
	nodeGeoNode.setMesh(nodeMesh);
	nodeGeoNode += stateNode;
	scene += nodeGeoNode;

	var lineMesh = Rendering.createLine(vd, path[i-1], path[i]);
	var lineGeoNode = new MinSG.GeometryNode();
	lineGeoNode.setMesh(lineMesh);
	lineGeoNode += stateNode;
	scene += lineGeoNode;
}

var nodeMesh = Rendering.createCube();
var nodeGeoNode = new MinSG.GeometryNode();
nodeGeoNode.moveLocal(path[path.count()-1]);
nodeGeoNode.scale(1);
nodeGeoNode.setMesh(nodeMesh);
nodeGeoNode += stateNode;
scene += nodeGeoNode;

var lineMesh = Rendering.createLine(vd, path[path.count()-2], path[path.count()-1]);
var lineGeoNode = new MinSG.GeometryNode();
lineGeoNode.setMesh(lineMesh);
lineGeoNode += stateNode;
scene += lineGeoNode;


// Camera ----------------------------------------------------------------------
var camera = PADrend.getDolly();
camera.setWorldOrigin(new Geometry.Vec3(10,20,30));
camera.rotateToWorldDir(new Geometry.Vec3(-120,350,280));
