// Create a new scene
var scene = PADrend.createNewSceneRoot("new MinSG.ListNode");

// Activate the scene
PADrend.selectScene(scene);

// Create the ground -----------------------------------------------------------
var vd = new Rendering.VertexDescription;
vd.appendPosition3D(); // 3xfloat (x,y,z)
vd.appendNormalByte(); // 4xbyte (x,y,z,0)
vd.appendColorRGBAByte(); // 4xbyte (r,g,b,a)

var grid = Rendering.createHexGrid(vd, 30, 30, 32, 32);

var scale = 10;
var transform = (new Geometry.Matrix4x4).scale(0.01, 0.01, 0.01); // scale matrix
Rendering.calculateNormals(grid);

var gridNode = new MinSG.GeometryNode(grid);

var stateGrid = new MinSG.MaterialState;
/// RGBA colors for ground
stateGrid.setAmbient(new Util.Color4f(0.1, 0.50, 0.2, 0.8));
stateGrid.setDiffuse(new Util.Color4f(0.1, 0.50, 0.2, 0.8));

gridNode += stateGrid;
scene += gridNode;

// Create obstacles (cubes) ----------------------------------------------------
var stateCube = new MinSG.MaterialState;
/// RGBA colors for first obstacle (cube)
stateCube.setAmbient(new Util.Color4f(0.46, 0.31, 0.23, 1));
stateCube.setDiffuse(new Util.Color4f(0.46, 0.31, 0.23, 1));

var cubeMesh = Rendering.createCube();
var geoNode = new MinSG.GeometryNode();
geoNode.moveLocal(10, 3, 8);
geoNode.scale(6);

geoNode.setMesh(cubeMesh);
geoNode += stateCube;
scene += geoNode;

var stateCube2 = new MinSG.MaterialState;
/// RGBA colors for second obstacle (cube)
stateCube2.setAmbient(new Util.Color4f(0.44, 0.38, 0.18, 1));

var cubeMesh2 = Rendering.createCube();
var geoNode2 = new MinSG.GeometryNode();
geoNode2.moveLocal(9, 7, 8);
geoNode2.scale(3);
geoNode2.setMesh(cubeMesh2);
geoNode2 += stateCube2;
scene += geoNode2;

// Create ramp using a rectangular box -----------------------------------------
var stateRamp = new MinSG.MaterialState;
/// RGBA colors for the ramp
stateRamp.setAmbient(new Util.Color4f(0.5, 0.5, 0.5, 1));
stateRamp.setDiffuse(new Util.Color4f(0.5, 0.5, 0.5, 1));

// Define the dimensions of the ramp using a Box object
var rampBox = new Geometry.Box(new Geometry.Vec3(4, 0, 8), new Geometry.Vec3(6, 1, 2));  // Size and position

// Create the ramp mesh using the Box object
var rampMesh = Rendering.createBox(vd, rampBox);  // Use createBox with VertexDescription and Box

// Create geometry node for the ramp
var geoRampNode = new MinSG.GeometryNode();
geoRampNode.moveLocal(5, 0, 8);  // Adjust position to connect with the cube
geoRampNode.scale(1.5);  // Scale it to look like a ramp
geoRampNode.rotateLocal(30, new Geometry.Vec3(1, 0, 0));  // Rotate to create a sloped surface (30 degrees)

geoRampNode.setMesh(rampMesh);
geoRampNode += stateRamp;
scene += geoRampNode;


// Call the algorithm ----------------------------------------------------------
// Change start and goal vectors for pathfinding
var start = new Geometry.Vec3(2, 0, 40);
var goal = new Geometry.Vec3(30, 0, 2);

// Call the pathfinder function
var path = Pathfinder.findPath(scene, start, goal, 1);
Pathfinder.enableOutput();

if (path.count() == 0) {
    path = [start, goal];
}

// Camera ----------------------------------------------------------------------
var camera = PADrend.getDolly();
camera.setWorldOrigin(new Geometry.Vec3(20, 20, 40));  // Adjust height and distance for better perspective
camera.rotateToWorldDir(new Geometry.Vec3(0, 45, 0));  // Rotate to face the ramp and obstacles
