/**
 * @file
 * 
 * @brief This file implements all functionality declared in AABB.h.
 * 
 * @author Jan-Luca Hansel
 */

#include <iostream>
#include <memory>
#include <chrono>

#include <spdlog/spdlog.h>
#include <spdlog/sinks/basic_file_sink.h>
#include <spdlog/stopwatch.h>
#include <spdlog/fmt/chrono.h>

#include "AABB.h"
#include "Using.h"

namespace Pathfinder {

	AABB::AABB() : 
		min(Geometry::Vec3()), 
		max(Geometry::Vec3()) {}


	AABB::AABB(const Geometry::Vec3& min, const Geometry::Vec3& max) : 
		min(min), 
		max(max) {}


	bool AABB::triNotInAABB(const Polygon* p, const short& component) {
		if((p->p1[component] < min[component] && 
		    p->p2[component] < min[component] && 
		    p->p3[component] < min[component]) || // All smaller
		   (p->p1[component] > max[component] && 
		    p->p2[component] > max[component] && 
		    p->p3[component] > max[component])) { // All greater
			return true;
		}
		return false;
	}


	bool AABB::triInAABB(const Polygon* p) {
		if(triNotInAABB(p, x) || triNotInAABB(p, y) || triNotInAABB(p, z)) {
			// This polygon cannot intersect
			return false;
		}
		return true;
	}


	bool AABB::pointInAABB(Geometry::Vec3 p) {
		if(min[x] < p[x] && min[y] < p[y] && min[z] < p[z] &&
		   max[x] > p[x] && max[y] > p[y] && max[z] > p[z]) {
			return true;
		}
		return false;
	}


	bool AABBintersection(const AABB& a, const AABB& b) {
		if(a.min[x] <= b.max[x] &&	// i.minX <= j.maxX
		   a.max[x] >= b.min[x] &&	// i.maxX >= j.minX
		   a.min[y] <= b.max[y] &&	// i.minY <= j.maxY
		   a.max[y] >= b.min[y] &&	// i.maxY >= j.minY
		   a.min[z] <= b.max[z] &&	// i.minZ <= j.maxZ
		   a.max[z] >= b.min[z]) {	// i.maxZ >= j.minZ
			return true;
		}
		return false;
	}

}
