/** 
 * @file
 *
 * @brief This file declares the sceneAnalysis function that is used to 
 *        perform the analysis of the scene. This analysis corresponds 
 *        to the first step described in the thesis.
 *
 * @author Jan-Luca Hansel
 */

#ifndef ANALYSIS_H
#define ANALYSIS_H

#include <MinSG/Core/Nodes/ListNode.h>

#include "Using.h"

namespace Pathfinder {

	/**
	 * @brief Create a directory where all output is written to
	 * @details This function will create a directory in ../Output.
	 *          The directory will be named after the current timestamp.
	 *
	 * @return Returns a string that contains the relative path to the 
	 *         created output directory.
	 */
	std::string createOutputDirs(); 


	/**
	 * @brief Perform the static analysis of the scene.
	 * @details Given the root node of a scene graph, get all GeometryNodes, 
	 *          build groups and the graphs of each group.
	 *
	 * @param root The root node of the scene.
	 *
	 * @return Returns a  vector of pointers to the graphs that were computed 
	 *         in the analysis.
	 */
	std::vector<Graph*>
	sceneAnalysis(
		const MinSG::ListNode& root
	);

}

#endif
