//
// Created by vinee on 7/25/24.
//

#include <AABB.h>
#include <Polygon.h>

#include "BoundingBox.h"
#include <Geometry/Matrix3x3.h>


namespace Pathfinder {

    bool isAxisAligned(const Polygon* polygon) {
        Geometry::Vec3 normal = polygon->normal;
        // Check if the normal is aligned with any of the primary axes
        return (normal.x() == 0 || normal.y() == 0 || normal.z() == 0);
    }

    BoundingBox* createBoundingBox(const Polygon* polygon) {
        if (isAxisAligned(polygon)) {
            // Use the min and max values from the polygon directly for the AABB
            return new AABB(polygon->min, polygon->max);
        } else {
            // For OBB, calculate center, half extents, and rotation matrix
            Geometry::Vec3 center = polygon->center;
            Geometry::Vec3 halfExtents(
                (polygon->max.x() - polygon->min.x()) / 2,
                (polygon->max.y() - polygon->min.y()) / 2,
                (polygon->max.z() - polygon->min.z()) / 2
            );
            // For the sake of this example, we'll assume the rotation is identity
            // You should calculate the actual rotation matrix based on polygon's orientation
            Geometry::_Matrix3x3<float> rotation; // Use the default constructor to set identity
            rotation.setIdentity();
            return new OBB(center, halfExtents, rotation);
        }
    }
}
