/** 
 * @file
 *
 * @brief This file implements all functionality declared in Compare.h.
 *
 * @author Jan-Luca Hansel
 */

#include <cmath>
#include <limits>
#include <chrono>

#include "Compare.h"
#include "Using.h"

namespace Pathfinder {

	bool compareSmallerVec3(const Geometry::Vec3& l, const Geometry::Vec3& r) {

		if(compareSmallerFloat(l[x], r[x])) {
			return true;
		} else if(compareEqualFloat(l[x], r[x])) {
			if(compareSmallerFloat(l[y], r[y])) {
				return true;
			} else if(compareEqualFloat(l[y], r[y])) {
				if(compareSmallerFloat(l[z], r[z])) {
					return true;
				}
			}
		}
		return false;
	}


	bool compareEqualVec3(const Geometry::Vec3& l, const Geometry::Vec3& r) {
		return (compareEqualFloat(l[x], r[x]) &&
				compareEqualFloat(l[y], r[y]) &&
				compareEqualFloat(l[z], r[z]));
	}


	bool compareSmallerEqualVec3(
		const Geometry::Vec3& l, const Geometry::Vec3& r
	) {
		return compareSmallerVec3(l, r) || compareEqualVec3(l, r);
	}


	bool compareSmallerVec3Tuple(
		const std::tuple<Geometry::Vec3, Geometry::Vec3>& l, 
		const std::tuple<Geometry::Vec3, Geometry::Vec3>& r
	) {
		if(compareSmallerVec3(std::get<0>(l), std::get<0>(r))) {
			return true;
		} else if(compareEqualVec3(std::get<0>(l), std::get<0>(r))) {
			if(compareSmallerVec3(std::get<1>(l), std::get<1>(r))) {
				return true;
			} 
		}
		return false;
	}


	bool compareEqualVec3Tuple(
		const std::tuple<Geometry::Vec3, Geometry::Vec3>& l, 
		const std::tuple<Geometry::Vec3, Geometry::Vec3>& r
	) {
		return (compareEqualVec3(std::get<0>(l), std::get<0>(r)) && 
				compareEqualVec3(std::get<1>(l), std::get<1>(r)));
	}


	bool compareSmallerVec3TupleTuple(
		const std::tuple<std::tuple<Geometry::Vec3, Geometry::Vec3>, 
						 std::tuple<Geometry::Vec3, Geometry::Vec3>>& l, 
		const std::tuple<std::tuple<Geometry::Vec3, Geometry::Vec3>, 
						 std::tuple<Geometry::Vec3, Geometry::Vec3>>& r
	) {
		if(compareSmallerVec3Tuple(std::get<0>(l), std::get<0>(r))) {
			return true;
		} else if(compareEqualVec3Tuple(std::get<0>(l), std::get<0>(r))) {
			if(compareSmallerVec3Tuple(std::get<1>(l), std::get<1>(r))) {
				return true;
			} 
		}
		return false;
	}


	bool compareEqualVec3TupleTuple(
		const std::tuple<std::tuple<Geometry::Vec3, Geometry::Vec3>, 
						 std::tuple<Geometry::Vec3, Geometry::Vec3>>& l, 
		const std::tuple<std::tuple<Geometry::Vec3, Geometry::Vec3>, 
						 std::tuple<Geometry::Vec3, Geometry::Vec3>>& r
	) {
		return (compareEqualVec3Tuple(std::get<0>(l), std::get<0>(r)) && 
				compareEqualVec3Tuple(std::get<1>(l), std::get<1>(r)));
	}

	
	bool compareEqualFloat(const float& l, const float& r) {
		return std::abs(l - r) <= std::numeric_limits<float>::epsilon();
	}

	
	bool compareSmallerFloat(const float& l, const float& r) {
		return (compareEqualFloat(l, r) ? false : (l < r));
	}
}
