/** 
 * @file
 *
 * @copyright Any copyright is dedicated to the Public Domain. 
 *            http://creativecommons.org/publicdomain/zero/1.0/ 
 *            Author: Sascha Brandt <sascha@brandt.graphics> 
 *
 * @brief This file is used to register C++ functions to EScript.
 *
 * @author Sascha Brandt 
 */

#include <iostream>

#include <EScript/Basics.h>
#include <EScript/StdObjects.h>

#include <E_MinSG/Core/Nodes/E_ListNode.h>
#include <E_Geometry/E_Vec3.h>

#include "EScriptHelper.h"
#include "Analysis.h"
#include "Search.h"
#include "Output.h"

namespace Pathfinder {

	/**
	 * @brief Initialize EScript bindings.
	 */
	static void initPathfinder(EScript::Namespace* lib) {	
		ES_FUNCTION(lib, "findPath", 4, 4, {
			MinSG::ListNode* lstNode = parameter[0].to<MinSG::ListNode*>(rt);
			Geometry::Vec3* start = parameter[1].to<Geometry::Vec3*>(rt);
			Geometry::Vec3* goal = parameter[2].to<Geometry::Vec3*>(rt);
			EScript::Number* agent = parameter[3].to<EScript::Number*>(rt);
			return findPath(*lstNode, *start, *goal, ((short) agent->getValue()));
		})

		ES_FUNCTION(lib, "sceneAnalysis", 1, 1, {
			MinSG::ListNode* lstNode = parameter[0].to<MinSG::ListNode*>(rt);
			sceneAnalysis(*lstNode);
			return EScript::value(nullptr);
		})

		ES_FUNCTION(lib, "disableOutput", 0, 0, {
			disableOutput();
			return EScript::value(nullptr);
		})

		ES_FUNCTION(lib, "enableOutput", 0, 0, {
			enableOutput();
			return EScript::value(nullptr);
		})

		ES_FUNCTION(lib, "disableLogging", 0, 0, {
			disableLogging();
			return EScript::value(nullptr);
		})

		ES_FUNCTION(lib, "enableLogging", 0, 0, {
			enableLogging();
			return EScript::value(nullptr);
		})

		ES_FUNCTION(lib, "disableOBJ", 0, 0, {
			disableOBJ();
			return EScript::value(nullptr);
		})
	}

	// Register the initializer method
	static bool initRegistered = registerEScriptInit(initPathfinder);

}
