/**
 * @file
 *
 * @brief This file declares the struct GraphVertex that contains all 
 *        information that shall be stored for each vertex of a graph. 
 *
 * @author Jan-Luca Hansel
 */

#ifndef GRAPHVERTEX_H
#define GRAPHVERTEX_H

#include "MeshAccessor.h"

namespace Pathfinder{

	/**
	 * @brief This struct declares all infroamtion that shall be stored for
	 *        each vertex of a graph.
	 */
	struct GraphVertex {

		/**
		 * The position of this vertex in three-dimensional space.
		 */
		Geometry::Vec3 pos; 


		/** 
		 * A pointer to the MeshAccessor object that manages the mesh from where 
		 * this vertex originates.
		 */
		MeshAccessor* ma;

		/**
		 * To find out if there is an obstacle in path
		 */
		bool is_obstacle;
	};

}

#endif
