/** 
 * @file
 *
 * @brief This file implements all functionality declared in Line2.h.
 *
 * @author Jan-Luca Hansel
 */

#include "Line2.h"
#include "Compare.h"

namespace Pathfinder {

	Line2::Line2(const float& m, const float& b) : m(m), b(b), vertical(false) {}


	Line2::Line2(Geometry::Vec2& u, Geometry::Vec2& v) {
		if(v[x] != u[x]) {
			// Not a vertical line
			this->m = (v[y]-u[y])/(v[x]-u[x]);
			this->b = u[y]-u[x]*(boost::get<float>(this->m));
			this->vertical = false;
		} else {
			// Vertical line
			this->vertical = true;
			this->xpos = u[x];
		}
	}


	boost::optional<float> Line2::eval(const float& x) {
		if(!vertical) {
			return (*m) * x + (*b);
		} else {
			return boost::none;
		}
	}


	boost::optional<float> line2Intersection(const Line2& a, const Line2& b) {
		if(a.vertical && b.vertical) {
			return boost::none;
		} else if(a.vertical) {
			return a.xpos;
		} else if(b.vertical) {
			return b.xpos;
		} else if(compareEqualFloat(*a.m, *b.m)) {
			return boost::none;
		} else {
			return (*b.b - *a.b) / (*a.m - *b.m);
		}
		return boost::none;
	}

}
