/** 
 * @file
 *
 * @brief This file declares the struct Line3 that represents three-dimensional 
 *        line.
 *
 * @author Jan-Luca Hansel
 */

#ifndef LINE3_H
#define LINE3_H

#include "Using.h"

namespace Pathfinder {

	/**
	 * @brief This struct represents a 3D line of the form u = v + rw.
	 */
	struct Line3 {

		/** 
		 * The support vector.
		 */
		Geometry::Vec3 v; 


		/** 
		 * The direction vector.
		 */
		Geometry::Vec3 w; 
	

		/**
		 * @brief Evaluate the line function for a given r.
		 * @details Given an r, scale the direction vector by r and add the 
		 *          support vector to the result. The resulting vector is the 
		 *          corresponding point to r in respect to this line.
		 *
		 * @param r The modifier for the direction vector.
		 *
		 * @return Return the resulting vector pointing to a point on the line.
		 */
		Geometry::Vec3 
		eval(
			const float& r
		);
 
	};

}

#endif
