/** 
 * @file
 *
 * @brief This file declares the struct Line3Norm that inherits from Line3.
 *        Its a Line3 with a normalized direction vector.
 *
 * @author Jan-Luca Hansel
 */

#ifndef LINE3NORM_H
#define LINE3NORM_H

#include "Line3.h"

namespace Pathfinder {

	/**
	 * @brief This struct represents a line of the form u = v + rw 
	 *        where w is normalized to length 1.
	 */
	struct Line3Norm : Line3 {

		/**
		 * @brief A constructor that creates a line from two vectors.
		 *        The direction vector will be normalized.
		 *
		 * @param a The support vector.
		 * @param b The direction vector.
		 */
		Line3Norm(
			Geometry::Vec3 a, 
			Geometry::Vec3 b
		);

	};

}

#endif
