/** 
 * @file
 *
 * @brief This file is needed to register the plugin to PADrend.
 *
 * @author Sascha Brandt
 */

/***************************************************\
* Any copyright is dedicated to the Public Domain.  *
* http://creativecommons.org/publicdomain/zero/1.0/ *
* Author: Sascha Brandt <sascha@brandt.graphics>    *
\***************************************************/

//============================================
// Do not modify this file!!!
//============================================

#include <iostream>

#include <EScript/EScript.h>

#include "EScriptHelper.h" 

#ifndef LIBRARY_NAME
#define LIBRARY_NAME "Pathfinder" // Will be set by CMake
#endif

/**
 * @brief Get the static registry for all init-functions.
 */
static std::vector<Pathfinder::EScriptInitFunction_t>& getInitRegistry() {
	static std::vector<Pathfinder::EScriptInitFunction_t> registry;
	return registry;
}

/**
 * @brief Implementation of the 'registerEScriptInit' function declared in 
 * 'EScriptHelper.h'.
 */
bool Pathfinder::registerEScriptInit(const EScriptInitFunction_t& initFn) {
	getInitRegistry().emplace_back(initFn);
	return true;
}

/**
 * @brief Main entry point for this library.
 * @details This function is called from EScript using the 
 *          'LibUtilExt/LibraryLoader' module. The 'lib' parameter is usually 
 *          the GLOBALS namespace from EScript.
 *
 * The 'extern "C"' signalizes the compiler that it should tread this function 
 * as a plain C function which can be easily found during dynamic linking 
 * (otherwise the compiler mangles the name).
 */
extern "C" void initLibrary(EScript::Namespace* lib) {
	std::cout << "Loading Library: " << LIBRARY_NAME << "..." << std::endl;
	if(lib->getAttribute(LIBRARY_NAME).isNotNull()) {
		std::cerr << LIBRARY_NAME << " Library already loaded!" << std::endl;
	} else {
		// create a new namespace under the GLOBALS namespace with the name of 
		// this library
		auto * ns = new EScript::Namespace;
		declareConstant(lib, LIBRARY_NAME, ns);
		
		// Initialize EScript
		for(auto& initFn : getInitRegistry())
			initFn(ns);
	}
}
