//
// Created by vinee on 6/26/24.
//
#ifndef OBB_H
#define OBB_H

#include <array>
#include <vector>
#include "BoundingBox.h"
#include "Vec.h"
#include "Polygon.h"
#include <Geometry/Matrix3x3.h>

    namespace Pathfinder {
    /**
     * @brief This struct models an OBB.
     * @details An OBB is defined to be the volume between the center and the
     *          half-extents in local space, rotated by a rotation matrix.
     */

 //   class OBB : public BoundingBox {
    struct OBB {
    public:
        //Geometry::Vec3 center;                          // Center of the OBB
        //Geometry::Vec3 halfSizes;                       // Half sizes (extents) of the OBB along each local axis
        std::array<Geometry::Vec3, 3> orientation;      // Local axes (normalized vectors)

        // Constructors
        OBB();
        OBB(const Geometry::Vec3& center, const Geometry::Vec3& halfExtents, const Geometry::_Matrix3x3<float>& rotation);

        Geometry::Vec3 getCenter() const ;

        Geometry::Vec3 getHalfExtents() const;

        Geometry::_Matrix3x3<float> getRotation() const;
        // Methods
        bool triNotInOBB(const Polygon* p, const short& component);              // Check if a triangle is not in the OBB using SAT separating axis theorem
        bool triInOBB(const Polygon* p);                                         // Check if a triangle intersects with the OBB
        bool pointInOBB(Geometry::Vec3 p);                                       // Check if a point is inside the OBB
        bool OBBintersection(const OBB& a, const OBB& b);                 // Check if two OBBs intersect using SAT

    private:
        Geometry::Vec3 center;
        Geometry::Vec3 halfExtents;
        Geometry::_Matrix3x3<float> rotation;
        static float dot(const Geometry::Vec3& a, const Geometry::Vec3& b);      // Dot product of two vectors
        static Geometry::Vec3 cross(const Geometry::Vec3& a, const Geometry::Vec3& b);  // Cross product of two vectors
        static void computeCovariance(const std::vector<Geometry::Vec3>& points, float covariance[3][3]);
        static void eigenDecomposition(const float covariance[3][3], Geometry::Vec3 eigenvalues, Geometry::Vec3 eigenvectors[3]);

        bool isAxisSeparating(const OBB &a, const OBB &b, const Geometry::Vec3 &axis);

        float projectOBB(const OBB &obb, const Geometry::Vec3 &axis);

        std::vector<Geometry::Vec3> getPointsFromObject();
    };

    }

#endif // OBB_H