/**
 * @file
 *
 * @brief This file implements all functionality declared in Output.h.
 *
 * @author Jan-Luca Hansel
 */

#include <iostream>

#include <spdlog/spdlog.h>
#include <spdlog/sinks/basic_file_sink.h>
#include <spdlog/stopwatch.h>
#include <spdlog/fmt/chrono.h>

#include "Output.h"

namespace Pathfinder {

	bool objOutput = true;

	void disableOutput() {
		std::cout.setstate(std::ios_base::failbit);
		std::cerr.setstate(std::ios_base::failbit);
		std::clog.setstate(std::ios_base::failbit);
	}


	void enableOutput() {
		std::cout.clear();
		std::cerr.clear();
		std::clog.clear();
	}


	void disableLogging() {
		spdlog::set_level(spdlog::level::off);
	}

	
	void enableLogging() {
		spdlog::set_level(spdlog::level::debug);
	}


	void createLoggers(const std::string& dir) {
		auto time_logger = 
			spdlog::basic_logger_mt("time_log", dir + "/times");
		time_logger->set_pattern("[time] %v");

		auto node_logger = 
			spdlog::basic_logger_mt("node_log", dir + "/nodes");
		node_logger->set_pattern("%v");

		auto warning_logger = 
			spdlog::basic_logger_mt("warn_log", dir + "/warnings");
		warning_logger->set_pattern("[warning] %v");
	}

	
	void disableOBJ() {
		objOutput = false;
	}

}
