/**
 * @file
 * 
 * @brief This file declares all functionality related to toggling output.
 *
 * @author Jan-Luca Hansel
 */

#ifndef OUTPUT_H
#define OUTPUT_H

#include <string>

namespace Pathfinder {

	/**
	 * Indicate whether OBJ files shall be created.
	 */
	extern bool objOutput; 

	
	/**
	 * @brief Prevents any output from being printed onto the terminal.
	 */
	void disableOutput(); 


	/**
	 * @brief Permits all output to be printed onto the terminal.
	 */
	void enableOutput(); 


	/**
	 * @brief Prevents the creation of log files.
	 */
	void disableLogging(); 

	
	/**
	 * @brief Permits the creation of log files.
	 */
	void enableLogging();

	
	/**
	 * @brief Create all logger objects used throughout the code.
	 *
	 * @param dir The directory where the log files shall be written to.
	 */
	void createLoggers(const std::string& dir);


	/**
	 * @brief Prevent any OBJ files from being created.
	 */
	void disableOBJ(); 

}

#endif
