/** 
 * @file
 *
 * @brief This file declares the struct Plane that represents a plane in 
 *        three-dimensional space.
 *
 * @author Jan-Luca Hansel
 */

#ifndef PLANE_H
#define PLANE_H

#include <Geometry/Vec3.h>

namespace Pathfinder {

	/**	
	 * @brief This struct models a plane of the form ax + by + cz + d = 0.
	 */
	struct Plane {
	
		/** 
		 * Parameter a of the plane.
		 */
		float a; 


		/** 
		 * Parameter b of the plane.
		 */
		float b; 


		/** 
		 * Parameter c of the plane.
		 */
		float c; 


		/** 
		 * Parameter d of the plane.
		 */
		float d; 


		/**
		 * @brief A basic constructor that creates a plane from four parameters.
		 *
		 * @param a Parameter a of the plane.
		 * @param b Parameter b of the plane.
		 * @param c Parameter c of the plane.
		 * @param d Parameter d of the plane.
		 */
		Plane(
			const float& a, 
			const float& b, 
			const float& c, 
			const float& d
		);

	
		/**
		 * @brief A basic constructor that creates a plane from three vectors.
		 * 
		 * @param u The first vector defining the plane.
		 * @param v The second vector defining the plane.
		 * @param w The third vector defining the plane.
		 */
		Plane(
			const Geometry::Vec3& u, 
			const Geometry::Vec3& v, 
			const Geometry::Vec3& w
		);

	};

}

#endif
