/**
 * @file
 *
 * @brief This file implements all functionality declared in Sort.h.
 * 
 * @author Jan-Luca Hansel
 */

#include <algorithm>
#include <vector>
#include <memory>
#include <chrono>

#include <spdlog/spdlog.h>
#include <spdlog/sinks/basic_file_sink.h>
#include <spdlog/stopwatch.h>
#include <spdlog/fmt/chrono.h>

#include "Compare.h"
#include "Sort.h"

namespace Pathfinder {

	void sortSizeTTuple(std::tuple<size_t, size_t, size_t>& t) {
		// Start timer for this function
		spdlog::stopwatch sw;

		std::vector<size_t> v = {std::get<0>(t), std::get<1>(t), std::get<2>(t)};
		std::sort(v.begin(), v.end());
		t = std::tuple<size_t, size_t, size_t>(v[0], v[1], v[2]);
		spdlog::get("time_log")->info("sortSizeTTuple: {}", std::chrono::duration_cast<std::chrono::microseconds>(sw.elapsed()));
	}


	void sortVec3Tuple(std::tuple<Geometry::Vec3, Geometry::Vec3>& t) {
		// Start timer for this function
		spdlog::stopwatch sw;

		Geometry::Vec3 a = std::get<0>(t);
		Geometry::Vec3 b = std::get<1>(t);
		// Switch a and b if b < a
		if(compareSmallerVec3(b, a)) {
			std::tuple<Geometry::Vec3, Geometry::Vec3> tNew(b, a);
			swap(t, tNew);
		}
		spdlog::get("time_log")->info("sortVec3Tuple: {}", std::chrono::duration_cast<std::chrono::microseconds>(sw.elapsed()));
	}


	void sortVec3TupleTuple(
		std::tuple<
			std::tuple<Geometry::Vec3, Geometry::Vec3>, 
			std::tuple<Geometry::Vec3, Geometry::Vec3>
		>& t
	) {
		// Start timer for this function
		spdlog::stopwatch sw;

		std::tuple<Geometry::Vec3, Geometry::Vec3> a = std::get<0>(t);
		std::tuple<Geometry::Vec3, Geometry::Vec3> b = std::get<1>(t);
		// Switch a and b if b < a
		if(compareSmallerVec3Tuple(b, a)) {
			std::tuple<std::tuple<Geometry::Vec3, Geometry::Vec3>, 
					   std::tuple<Geometry::Vec3, Geometry::Vec3>> tNew(b, a);
			swap(t, tNew);
		}
		spdlog::get("time_log")->info("sortVec3TupleTuple: {}", std::chrono::duration_cast<std::chrono::microseconds>(sw.elapsed()));
	}

}
