/** 
 * @file
 *
 * @brief This file implements all functionality declared in TupleAccess.h.
 *
 * @author Jan-Luca Hansel
 */

#include <spdlog/spdlog.h>
#include <spdlog/sinks/basic_file_sink.h>
#include <spdlog/stopwatch.h>
#include <spdlog/fmt/chrono.h>

#include "TupleAccess.h"

namespace Pathfinder {

	float gettValue(
		const std::tuple<float, float, float, float, float, float>& tValues, 
		const short& idx
	) {
		switch(idx) {
			case 0: return std::get<0>(tValues);
			case 1: return std::get<1>(tValues);
			case 2: return std::get<2>(tValues);
			case 3: return std::get<3>(tValues);
			case 4: return std::get<4>(tValues);
			case 5: return std::get<5>(tValues);
			default: spdlog::get("warn_log")->info("gettValue returns -1.");
					 return -1;
		}
	}


	std::vector<short> gettValueIndexCategoryVector(
		const std::tuple<std::vector<short>, 
						 std::vector<short>, 
						 std::vector<short>, 
						 std::vector<short>, 
						 std::vector<short>>& categorizedtValueIndices, 
		const short& idx
	) {
		switch(idx) {
			case 0: return std::get<0>(categorizedtValueIndices);
			case 1: return std::get<1>(categorizedtValueIndices);
			case 2: return std::get<2>(categorizedtValueIndices);
			case 3: return std::get<3>(categorizedtValueIndices);
			case 4: return std::get<4>(categorizedtValueIndices);
			default: spdlog::get("warn_log")->info("gettValueIndexCategoryVector returns \{}.");
					 return {};
		}
	}


	Line2 getLine2(
		const std::tuple<Line2, Line2, Line2>& lines, 
		const short& idx
	) {
		switch(idx) {
			case 0: return std::get<0>(lines);
			case 1: return std::get<1>(lines);
			case 2: return std::get<2>(lines);
			default: spdlog::get("warn_log")->info("getLine2 returns Line2(0, 0).");
					 return Line2(0, 0);
		}
	}


	bool* getBoolPtr(
		std::tuple<bool, bool, bool>& boolTuple, 
		const short& idx
	) {
		switch(idx) {
			case 0: return &std::get<0>(boolTuple);
			case 1: return &std::get<1>(boolTuple);
			case 2: return &std::get<2>(boolTuple);
			default: spdlog::get("warn_log")->info("getBoolPtr returns nullptr.");
					 return nullptr;
		}
	}


	short* getShortPtr(
		std::tuple<short, short, short>& shortTuple, 
		const short& idx
	) {
		switch(idx) {
			case 0: return &std::get<0>(shortTuple);
			case 1: return &std::get<1>(shortTuple);
			case 2: return &std::get<2>(shortTuple);
			default: spdlog::get("warn_log")->info("getShortPtr returns nullptr.");
					 return nullptr;
		}
	}


	boost::optional<float> getOptionalFloat(
		const std::tuple<boost::optional<float>, 
						 boost::optional<float>, 
						 boost::optional<float>>& optionalFloatTuple, 
		const short& idx
	) {
		switch(idx) {
			case 0: return std::get<0>(optionalFloatTuple);
			case 1: return std::get<1>(optionalFloatTuple);
			case 2: return std::get<2>(optionalFloatTuple);
			default: spdlog::get("warn_log")->info("getoptionalFloat returns boost::none.");
					 return boost::none;
		}
	}


	bool getBool(
		const std::tuple<bool, bool, bool>& boolTuple,
		const short& idx
	) {
		switch(idx) {
			case 0: return std::get<0>(boolTuple);
			case 1: return std::get<1>(boolTuple);
			case 2: return std::get<2>(boolTuple);
			default: spdlog::get("warn_log")->info("getBool returns false.");
					 return false;
		}
	}


	Geometry::Vec3 getTriVertex(
		const std::tuple<Geometry::Vec3, Geometry::Vec3, Geometry::Vec3>& tri, 
		const short& idx
	) {
		switch(idx) {
			case 0: return std::get<0>(tri);
			case 1: return std::get<1>(tri);
			case 2: return std::get<2>(tri);
			default: spdlog::get("warn_log")->info("getTriVertex returns \{}.");
					 return {};
		}	
	}
	

	Geometry::Vec2 getFlatTriVertex(
		const std::tuple<Geometry::Vec2, Geometry::Vec2, Geometry::Vec2>& tri, 
		const short& idx
	) {
		switch(idx) {
			case 0: return std::get<0>(tri);
			case 1: return std::get<1>(tri);
			case 2: return std::get<2>(tri);
			default: spdlog::get("warn_log")->info("getFlatTriVertex returns \{}.");
					 return {};
		}	
	}


}
