/** 
 * @file
 *
 * @brief This file provides using and enum declarations.
 *
 * @author Jan-Luca Hansel
 */

#ifndef USING_H
#define USING_H

#include <list>
#include <unordered_map>
#include <string>

#include <boost/graph/adjacency_list.hpp>

#include "GraphVertex.h"
#include "GraphEdge.h"

namespace Pathfinder {

	/**
	 * This indicates the passability of edges.
	 */
	enum drivable : short {person, bicycle, car, truck, none};	


	/**
	 * Easier notation of coordinates.
	 */
	enum coordinate : short {x, y, z}; 


	/**
	 * Shorthand notation for the graph used.
	 */
    /**using Graph = boost::adjacency_list<boost::vecS,
                                        boost::vecS,
                                        boost::undirectedS,
                                        GraphVertex,
                                        GraphEdge>;
        */
        /* * @author Vineet */
   /* Converting undirected graph to directed graph*/
    using Graph = boost::adjacency_list<boost::vecS,
                                        boost::vecS,
                                        boost::directedS,
                                        GraphVertex,
                                        GraphEdge>;

	/**
	 * Shorthand notation for the Graph Connected.
	 */

	bool isGraphConnected(const Graph& graph);


    /**
     * Shorthand notation for the graph vertices.
     */
	using Vertex = boost::graph_traits<Graph>::vertex_descriptor;


	/**
	 * Shorthand notation for the graph edges.
	 */
	using Edge = boost::graph_traits<Graph>::edge_descriptor;


	/**
	 * Color definition for prettier console output.
	 * 
	 * @copyright Taken from: https://stackoverflow.com/questions/2616906/how-do-i-output-coloured-text-to-a-linux-terminal
	 */
	const std::string red("\033[0;31m");


	/**
	 * Color definition for prettier console output.
	 * 
	 * @copyright Taken from: https://stackoverflow.com/questions/2616906/how-do-i-output-coloured-text-to-a-linux-terminal
	 */
	const std::string green("\033[1;32m");


	/**
	 * Color definition for prettier console output.
	 * 
	 * @copyright Taken from: https://stackoverflow.com/questions/2616906/how-do-i-output-coloured-text-to-a-linux-terminal
	 */
	const std::string yellow("\033[1;33m");


	/**
	 * Color definition for prettier console output.
	 * 
	 * @copyright Taken from: https://stackoverflow.com/questions/2616906/how-do-i-output-coloured-text-to-a-linux-terminal
	 */
	const std::string cyan("\033[0;36m");


	/**
	 * Color definition for prettier console output.
	 * 
	 * @copyright Taken from: https://stackoverflow.com/questions/2616906/how-do-i-output-coloured-text-to-a-linux-terminal
	 */
	const std::string magenta("\033[0;35m");


	/**
	 * Color definition for prettier console output.
	 * 
	 * @copyright Taken from: https://stackoverflow.com/questions/2616906/how-do-i-output-coloured-text-to-a-linux-terminal
	 */
	const std::string reset("\033[0m");

}

#endif
